/*!
  \file Draw_DDALine.c
  \author Mario Palomo <mpalomo@ihman.com>
  \author Jose M. de la Huerga Fernndez
  \author Pepe Gonzlez Mora
  \date 05-2002

  Based in Kenny Hoff sourcer.
  Purpose: To implement a generalized Digital Differential Analyzer (DDA) line drawing
           algorithm, to compare it to my generalized (covering all slopes) Bresenham's
           version.
  Notes:	This algorithm was basically taken from Hearn and Baker's "Computer Graphics"
           (pg 88). However, I modified it to handle the length-of-one case, to
            perform only integer calculations, and to require one less variable in the
            loop. However, I did have to add two new variables in initialization to
            get rid of the repetitive calls to abs() (which I think is only a macro
            anyway).

*/
/*
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/


#define SDL_DRAW_PUTPIXEL_BPP(A, B, C)  \
*(##A##(##B##(Uint8*)super->pixels + (CurrentY>>6)*super->pitch + (CurrentX>>6)*SDL_DRAW_BPP))=##C##;

#if SDL_DRAW_BPP == 1
#define SDL_DRAW_PUTPIXEL SDL_DRAW_PUTPIXEL_BPP(0+,0+,color)

#elif SDL_DRAW_BPP == 2
#define SDL_DRAW_PUTPIXEL SDL_DRAW_PUTPIXEL_BPP((Uint16*),0+,color)

#elif SDL_DRAW_BPP == 3
#define SDL_DRAW_PUTPIXEL \
  SDL_DRAW_PUTPIXEL_BPP(0+,1+,(Uint8)colorbyte1)   \
  if (SDL_BYTEORDER == SDL_BIG_ENDIAN) {         \
    SDL_DRAW_PUTPIXEL_BPP(0+,0+,(Uint8)colorbyte2)   \
    SDL_DRAW_PUTPIXEL_BPP(0+,2+,(Uint8)colorbyte0) \
  }else{                                         \
    SDL_DRAW_PUTPIXEL_BPP(0+,0+,(Uint8)colorbyte0)   \
    SDL_DRAW_PUTPIXEL_BPP(0+,2+,(Uint8)colorbyte2) \
  }

#elif SDL_DRAW_BPP == 4
#define SDL_DRAW_PUTPIXEL SDL_DRAW_PUTPIXEL_BPP((Uint32*),0+,color)

#endif /*SDL_DRAW_BPP*/


#define ABS(X) (X < 0) ? -X : X;

void SDL_DRAWFUNCTION(SDL_Surface *super,
                      Sint16 x1, Sint16 y1, Sint16 x2, Sint16 y2,
                      Uint32 color)
{
#if SDL_DRAW_BPP == 3
  Uint8 colorbyte0 = (Uint8) (color & 0xff);
  Uint8 colorbyte1 = (Uint8) ((color >> 8) & 0xff);
  Uint8 colorbyte2 = (Uint8) ((color >> 16) & 0xff);
#endif
  Sint16 dX, dY;
  Sint16 x, y;
  register Sint16 CurrentX; 
  register Sint16 CurrentY;
  register Sint16 NumSteps;
  Sint16 AbsDx;
  Sint16 AbsDy;
 
  /* Clip line and test if we have to draw, only if we need to do it */
#ifdef SDL_DRAW_CLIP
  if (!(clipLine(super, &x1, &y1, &x2, &y2))) { return; }
#endif

 /* Lock surface */
  if (SDL_MUSTLOCK(super)) {
      if (SDL_LockSurface(super) < 0)  { return; }
  }

  
  dX = x2-x1;
  dY = y2-y1;
  AbsDx = abs(dX);
  AbsDy = abs(dY);

  if (AbsDx > AbsDy) { NumSteps = AbsDx; }
  else { NumSteps = AbsDy; } 

  if (NumSteps == 0){ 	
    CurrentX=x1;
    CurrentY=y1;
    SDL_DRAW_PUTPIXEL 	
  }else{	

    Sint16 Xincr = (dX<<6) / NumSteps;
    Sint16 Yincr = (dY<<6) / NumSteps;
    
    CurrentX = x1<<6;
    CurrentY = y1<<6;
  
    for (; NumSteps >= 0; NumSteps--){
       SDL_DRAW_PUTPIXEL
       CurrentX += Xincr;
       CurrentY += Yincr;
     }
  }

  /* Unlock surface */
  if (SDL_MUSTLOCK(super))  { SDL_UnlockSurface(super); }
 

}/*Draw_DDALine*/


#undef SDL_DRAW_PUTPIXEL
#undef SDL_DRAW_PUTPIXEL_BPP

